@@ -1,4 +1,4 @@
-// This file Copyright © 2007-2022 Mnemosyne LLC.
+// This file Copyright © 2007-2023 Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -43,6 +43,7 @@
 #include "session.h"
 #include "timer.h"
 #include "torrent.h"
+#include "torrent-magnet.h"
 #include "tr-assert.h"
 #include "tr-utp.h"
 #include "utils.h"
@@ -644,7 +645,12 @@ public:
 
     tr_torrent* const tor;
 
+    ActiveRequests active_requests;
+
+    // depends-on: active_requests
     std::vector<std::unique_ptr<tr_peer>> webseeds;
+
+    // depends-on: active_requests
     std::vector<tr_peerMsgs*> peers;
 
     // tr_peers hold pointers to the items in this container,
@@ -656,8 +662,6 @@ public:
 
     time_t lastCancel = 0;
 
-    ActiveRequests active_requests;
-
 private:
     static void maybeSendCancelRequest(tr_peer* peer, tr_block_index_t block, tr_peer const* muted)
     {
@@ -2388,8 +2392,11 @@ void tr_peerMgr::bandwidthPulse()
     session->top_bandwidth_.allocate(Msec);
 
     // torrent upkeep
-    auto& torrents = session->torrents();
-    std::for_each(std::begin(torrents), std::end(torrents), [](auto* tor) { tor->do_idle_work(); });
+    for (auto* const tor : session->torrents())
+    {
+        tor->do_idle_work();
+        tr_torrentMagnetDoIdleWork(tor);
+    }
 
     /* pump the queues */
     queuePulse(session, TR_UP);
